# LSB Test Execution Framework
# General Templates and Functions Module (Templates.pm)
#
# Copyright (C) 2007 The Linux Foundation. All rights reserved.
#
# This program has been developed by ISP RAS for LF.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #

package Templates;
use strict;
require Exporter;

@Templates::ISA = "Exporter";
@Templates::EXPORT = qw($DOCUMENT_ROOT $APP_DATA %_GET %_POST %_COOKIE %CONFIG $PeerIP &print_header &print_footer &autoflush_on &escape &unescape);

my ($global_header, $global_footer);

our $DOCUMENT_ROOT = $0;
$DOCUMENT_ROOT =~ s/\/[^\/]+$//;

our $APP_DATA = "/var/opt/lsb/app-testkit/manager";

if (open(FILE, "$DOCUMENT_ROOT/header.tpl")) {
	$global_header = "";
	while (<FILE>) {
		$global_header .= $_;
	}
	close(FILE);
}
if (open(FILE, "$DOCUMENT_ROOT/footer.tpl")) {
	$global_footer = "";
	while (<FILE>) {
		$global_footer .= $_;
	}
	close(FILE);
}
if (!$global_header or !$global_footer) {
	$global_header = "<html><body>\n";
	$global_footer = "</body></html>";
}

sub escape($) {
	$_[0] =~ s/([^0-9a-zA-Z_ ])/sprintf("%%%02X", ord($1))/eg;
	$_[0] =~ s/ /\+/g;
}

sub unescape($) {
	$_[0] =~ s/\+/ /g;
	$_[0] =~ s/%([0-9a-fA-F]{2})/chr(hex($1))/eg;
}

my $tmp;
my @args;

our %_GET = ();
$tmp = shift;
@args = split(/&/, $tmp);
foreach (@args) {
	my ($name, $val) = split(/=/, $_, 2);
	if ($name) {
#		unescape($name);
		if (defined($val)) {
			unescape($val);
		}
		else {
			$val = "1";
		}
		$_GET{$name} = $val;
	}
}

our %_POST = ();
$tmp = shift;
@args = split(/&/, $tmp);
foreach (@args) {
	my ($name, $val) = split(/=/, $_, 2);
	if ($name) {
#		unescape($name);
		if (defined($val)) {
			unescape($val);
		}
		else {
			$val = "1";
		}
		$_POST{$name} = $val;
	}
}

our %_COOKIE = ();
$tmp = shift;
my @cookies_list = split(/;\s*/, $tmp);
foreach (@cookies_list) {
	my ($name, $val) = split(/=/, $_, 2);
	if ($name and defined($val)) {
#		unescape($name);
		unescape($val);
		$_COOKIE{$name} = $val;
	}
}

# Get "$DOCUMENT_ROOT/../.." path
$tmp = $DOCUMENT_ROOT;
$tmp =~ s!/[^/]+/[^/]+$!!;
our %CONFIG = (
	"TESTS_DIR" => "/opt/lsb/bin",
	"UTILS_DIR" => "$tmp/utils",
	"RESULTS_DIR" => "$APP_DATA/results"
);

our $PeerIP = shift;

my %styles = (
	'conf' => '\$\$\$CONFIGURE_STYLE\$\$\$',
	'results' => '\$\$\$RESULTS_STYLE\$\$\$',
	'report' => '\$\$\$RESULTS_STYLE\$\$\$',
	'analysis' => '\$\$\$RESULTS_STYLE\$\$\$',
	'upload' => '\$\$\$RESULTS_STYLE\$\$\$',
	'help' => '\$\$\$HELP_STYLE\$\$\$',
	'about' => '\$\$\$ABOUT_STYLE\$\$\$',
	'admin' => '\$\$\$ADMIN_STYLE\$\$\$'
);

sub print_header($$) {
	$global_header =~ s/\$\$\$PAGE_TITLE\$\$\$/$_[0]/;
	# Find and highlight the active menu link
	foreach my $name (keys %styles) {
		if ($_[1] eq $name) {
			$global_header =~ s/$styles{$name}/-active/;
			last;
		}
	}
	# Set all other menu links to non-highlighted style
	foreach (values %styles) {
		$global_header =~ s/$_//;
	}
	if ($_[1]) {
		$global_header =~ s/\$\$\$HELP_REF\$\$\$/#$_[1]/;
	}
	else {
		$global_header =~ s/\$\$\$HELP_REF\$\$\$//;
	}
	print $global_header;
}

sub print_footer {
	if ($_[0]) {
		$global_footer =~ s/<\/body>//;
		$global_footer =~ s/<\/html>//;
	}
	print $global_footer;
}

# Remove old session profiles
my $tm = time();
if (opendir(DIR, "$APP_DATA/profiles")) {
	my @files = grep /^~session\./, readdir(DIR);
	foreach (@files) {
		my $name = "$APP_DATA/profiles/$_";
		# Session expiration time is 7 days
		if ($tm - (stat($name))[8] > 7*86400) {
			unlink($name);
		}
	}
	closedir(DIR);
}

sub autoflush_on() {
	my $old_handle = select (STDOUT);
	$| = 1;
	select (STDERR);
	$| = 1;
	select ($old_handle);
}

1;
