#!/usr/bin/perl -w

# LSB Test Execution Framework
# Configuration Save Module (tests_conf_save.pl)
#
# Copyright (C) 2007 The Linux Foundation. All rights reserved.
#
# This program has been developed by ISP RAS for LF.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #

use Templates;
use Fcntl ':flock';

sub is_process_finished($) {
	my $PID = $_[0];
	my $test_line = `ps -p $PID -o s --no-heading`;
	$test_line =~ s/[\r\n]//g;
	return ($test_line !~ m/^[^Zz]/);
}

# Helps to determine platform name.
sub guess_platform($) {
    if ($_[0] =~ /^(amd64|x86_64)$/) {
		return "x86_64";
	}
    elsif ($_[0] =~ /^(ia32|i[3-6]86|x86)$/) {
		return "ia32";
	}
    elsif ($_[0] =~ /^(ia64)$/) {
		return "ia64";
	}
    elsif ($_[0] =~ /^(ppc|ppc32)$/) {
		return "ppc32";
	}
    elsif ($_[0] =~ /^(ppc64)$/) {
		return "ppc64";
	}
    elsif ($_[0] =~ /^(s390)$/) {
		return "s390";
	}
    elsif ($_[0] =~ /^(s390x)$/) {
		return "s390x";
	}
    else {
		return "unknown";
	}
}

sub add_dir_to_list($$$) {
	my ($dir, $file_list, $recursive) = @_;
	if (opendir(DIR, $dir)) {
		foreach (grep(!/^\.+$/, readdir(DIR))) {
			if (-f "$dir/$_") {
				$file_list->{"$dir/$_"} = 1;
			}
			else {
				if ($recursive) {
					&add_dir_to_list("$dir/$_", $file_list, $recursive);
				}
			}
		}
		closedir(DIR);
	}
}

my $conf_page = "tests_conf.pl";
if ($_GET{"goto"}) {
	$conf_page = $_GET{"goto"};
	$conf_page =~ s/%23/#/;
	$_POST{"form_action"} = "";
}
my $session_profile = ($_COOKIE{"session_id"} ? "~session.$PeerIP.".$_COOKIE{"session_id"} : "");

my $error;
my $new_url = "";
my $save_profile_name = "";

if (!-d "$APP_DATA/profiles") {
	system("mkdir -p $APP_DATA/profiles");
	if (!-d "$APP_DATA/profiles") {
		$error = "";
	}
}

if (!defined($error)) {
	if ((!$_POST{"form_action"} or ($_POST{"form_action"} eq "")) and (!$_GET{"goto"})) {
		$error = "Script action not specified!";
	}
	elsif ($_POST{"form_action"} eq "load_profile") {
		if (!$_POST{"edit_profile_name"}) {
			$error = "No profile name specified!";
		}
		elsif ($_POST{"edit_profile_name"} =~ m/[\\\/]/) {
			$error = "Invalid profile name!";
		}
		elsif (!-f "$APP_DATA/profiles/".$_POST{"edit_profile_name"}) {
			$error = "File not found:<br />$APP_DATA/profiles/".$_POST{"edit_profile_name"};
		}
		else {
			system("cp \"$APP_DATA/profiles/$_POST{edit_profile_name}\" \"$APP_DATA/profiles/$session_profile\" > /dev/null");
			$new_url = $conf_page;
		}
	}
	elsif ($_POST{"form_action"} eq "run_tests") {
		if ($session_profile eq "") {
			$error = "Cannot retrieve session ID! Please, allow cookies in your browser.";
		}
		else {
			$save_profile_name = $session_profile;
			$new_url = "tests_results.pl";
		}
	}
	elsif ($_POST{"form_action"} eq "save_profile") {
		if (!$_POST{"edit_profile_name"}) {
			$error = "No profile name specified!";
		}
		elsif ($_POST{"edit_profile_name"} =~ m/[\\\/]/) {
			$error = "Invalid profile name!";
		}
		else {
			$save_profile_name = $_POST{"edit_profile_name"};
			$new_url = $conf_page;
		}
	}
	elsif ($_POST{"form_action"} eq "delete_profile") {
		if (!$_POST{"edit_profile_name"}) {
			$error = "No profile name specified!";
		}
		elsif ($_POST{"edit_profile_name"} =~ m/[\\\/]/) {
			$error = "Invalid profile name!";
		}
		elsif (!-f "$APP_DATA/profiles/".$_POST{"edit_profile_name"}) {
			$error = "File not found:<br />$APP_DATA/profiles/".$_POST{"edit_profile_name"};
		}
		else {
			unlink("$APP_DATA/profiles/".$_POST{"edit_profile_name"});
			$save_profile_name = $session_profile;
			$new_url = $conf_page;
		}
	}
	elsif ($_GET{"goto"}) {
		$save_profile_name = $session_profile;
		$new_url = $conf_page;
	}
}

if (!defined($error)) {
	if ($save_profile_name ne "") {
		my $full_save_name = "$APP_DATA/profiles/$save_profile_name";
		if (open(FILE, "> $full_save_name")) {
			my $architecture = ($_POST{"architecture"} or guess_platform(`uname -m`));
			my $lsb_version  = ($_POST{"lsb_version"} or "3.1");
			my $lsb_profile  = ($_POST{"lsb_profile"} or "desktop");
			print FILE "APPLICATION:     ".$_POST{"application"}."\n" if ($_POST{"application"});
			if ($_POST{"libraries"}) {
				$_POST{"libraries"} =~ s/\r\n/\n/g;
				my @libraries_lines = split(/\n/, $_POST{"libraries"});
				foreach (@libraries_lines) {
					print FILE "LIBRARY:         $_\n";
				}
			}
			print FILE "ARCHITECTURE:    $architecture\n";
			print FILE "LSB_VERSION:     $lsb_version\n";
			print FILE "LSB_PROFILE:     $lsb_profile\n";

			if ($_POST{"comments"}) {
				$_POST{"comments"} =~ s/\r\n/\n/g;
				$_POST{"comments"} =~ s/&/&amp;/g;
				$_POST{"comments"} =~ s/</&lt;/g;
				$_POST{"comments"} =~ s/>/&gt;/g;
				my @comments_lines = split(/\n/, $_POST{"comments"});
				foreach (@comments_lines) {
					print FILE "COMMENTS: $_\n";
				}
			}

			close(FILE);

			if ($_POST{"form_action"} eq "save_profile") {
				system("cp \"$full_save_name\" \"$APP_DATA/profiles/$session_profile\" > /dev/null");
			}
			elsif ($_POST{"form_action"} eq "run_tests") {
				if (!$_POST{"application"}) {
					$error = "Application is not specified.";
				}
				else {
					my $app_name = $_POST{"application"};
					$app_name =~ s/^.*\/([^\/]*)$/$1/;
					my $cmd_line = $CONFIG{"TESTS_DIR"}."/lsbappchk -r $lsb_version -j -o /tmp/$app_name.journal.$$ ";
					if ($lsb_profile eq "desktop") {
						$cmd_line .= "-Tcore,c++,desktop ";
					}
					elsif ($lsb_profile eq "core") {
						$cmd_line .= "-Tcore,c++ ";
					}
					my %file_list = ();
					my @dir_list = ();
					my $recursive_dirs = 0;
					if (-d $_POST{"application"}) {
						$recursive_dirs = 1;
						push @dir_list, $_POST{"application"};
					}
					else {
						$file_list{$_POST{"application"}} = 1;
					}
					if ($_POST{"libraries"}) {
						my @libraries_lines = split(/\n/, $_POST{"libraries"});
						foreach (@libraries_lines) {
							if (-d) {
								$cmd_line .= "-D$_ ";
								push @dir_list, $_;
							}
							else {
								$cmd_line .= "-L$_ ";
								$file_list{$_} = 1;
							}
						}
					}
					$cmd_line .= '"'.$_POST{"application"}.'"';
					if ($recursive_dirs) {
						system("touch /tmp/$app_name.journal.$$");
						system("touch /tmp/lsbappchk_test.$$");
					}
					else {
						system($cmd_line." >/tmp/lsbappchk_test.$$ 2>&1");
					}
					my ($sec, $min, $hour, $mday, $mon, $year, $wday, $yday, $isdst) = localtime(time);
					my $host_name = `uname -n`;
					$host_name =~ s/[\r\n]//g;
					my $results_dir = sprintf("$architecture-$host_name-$app_name-%04d-%02d-%02d-%02dh-%02dm-%02ds", ($year + 1900), ($mon + 1), $mday, $hour, $min, $sec);
					system("mkdir -p ".$CONFIG{"RESULTS_DIR"}."/$results_dir/test_journals");
					system("mkdir -p ".$CONFIG{"RESULTS_DIR"}."/$results_dir/test_logs");
					if ((!-d $CONFIG{"RESULTS_DIR"}."/$results_dir/test_journals") or (!-d $CONFIG{"RESULTS_DIR"}."/$results_dir/test_logs")) {
						$error = "Could not create subdirectories for storing the test results in<br />$results_dir<br />$!<br />The console output and the jounal file are located here:<br /><a href=\"get.pl/tmp/lsbappchk_test.$$\"><u>/tmp/lsbappchk_test.$$</u></a>, <a href=\"get.pl/tmp/$app_name.journal.$$\"><u>/tmp/$app_name.journal.$$</u></a>";
					}
					system("echo \"$results_dir\" >> ".$CONFIG{"RESULTS_DIR"}."/HISTORY");
					system("mv /tmp/$app_name.journal.$$ ".$CONFIG{"RESULTS_DIR"}."/$results_dir/test_journals/lsbappchk.$app_name.journal");
					system("mv /tmp/lsbappchk_test.$$ ".$CONFIG{"RESULTS_DIR"}."/$results_dir/test_logs/tests_output");
					if (open(INFO_FILE, ">$CONFIG{RESULTS_DIR}/$results_dir/test_journals/lsbappchk.$app_name.journal.INFO")) {
						print INFO_FILE "TITLE: lsbappchk/$app_name\nANCHOR: lsbappchk_$app_name\nJOURNAL_FILENAME: lsbappchk.$app_name.journal\n";
						close(INFO_FILE);
					}
					if (open(COMMENTS_FILE, ">$CONFIG{RESULTS_DIR}/$results_dir/test_logs/test_comments")) {
						print COMMENTS_FILE $_POST{"comments"};
						close(COMMENTS_FILE);
					}
					system("echo \"$lsb_version\" >> ".$CONFIG{"RESULTS_DIR"}."/$results_dir/test_logs/lsb_version");
					system("echo \"$architecture\" >> ".$CONFIG{"RESULTS_DIR"}."/$results_dir/test_logs/host_platform");

					# Now all preparations are done, generate the report
					if (-s $CONFIG{"RESULTS_DIR"}."/$results_dir/test_logs/tests_output" != 0) {
						system("echo \"\" >>".$CONFIG{"RESULTS_DIR"}."/$results_dir/test_logs/tests_output");
					}
					if ($recursive_dirs) {
						if (open(TEST_LOG, ">".$CONFIG{"RESULTS_DIR"}."/$results_dir/test_log.html")) {
							print TEST_LOG <<DATA;
<html>
<body>
A directory was specified instead of the application. The test was not executed, only analysis was performed.
<ul>
  <li><a href="tests_results.pl?details=$results_dir&amp;analysis=1">View the list of libraries and interfaces used.</a></li>
  <li><a href="upload_data.pl?details=$results_dir">Send information to the Linux Foundation database.</a></li>
</ul>
</body>
</html>
DATA
							close(TEST_LOG);
						}
						if (open(INFO_FILE, ">".$CONFIG{"RESULTS_DIR"}."/$results_dir/INFO")) {
							print INFO_FILE <<DATA;
0 Tests Total
0 Tests Passed
0 Failures
0 Tests Quelled
0 Tests Skipped
Verdict: Failed
Application: $app_name
DATA
						    close(INFO_FILE);
    					}
						system("cd $CONFIG{RESULTS_DIR}; tar czf lsbappchk-$results_dir.tgz $results_dir && mv lsbappchk-$results_dir.tgz $results_dir");
					}
					else {
						system($CONFIG{"UTILS_DIR"}."/report.pl ".$CONFIG{"RESULTS_DIR"}."/$results_dir -o lsbappchk-$results_dir.tgz >>".$CONFIG{"RESULTS_DIR"}."/$results_dir/test_logs/tests_output 2>&1");
					}

					# Generate application analysis data
					if (open(FILE_LIST, ">/tmp/atkm_list.$$")) {
						foreach $dir (@dir_list) {
							add_dir_to_list($dir, \%file_list, $recursive_dirs);
						}
						foreach (keys %file_list) {
							print FILE_LIST "$_\n";
						}
						close(FILE_LIST);
						# apptodb needs PM file located in its directory.
						chdir($CONFIG{"UTILS_DIR"});
						system("echo \"\" >>".$CONFIG{"RESULTS_DIR"}."/$results_dir/test_logs/tests_output");
						system("echo \"Collecting data about application files\" >>".$CONFIG{"RESULTS_DIR"}."/$results_dir/test_logs/tests_output");
						system($CONFIG{"UTILS_DIR"}."/apptodb.pl -a $architecture -l /tmp/atkm_list.$$ -o ".$CONFIG{"RESULTS_DIR"}."/$results_dir/${app_name}_${architecture}_upload_data $app_name >>".$CONFIG{"RESULTS_DIR"}."/$results_dir/test_logs/tests_output 2>&1");
						unlink("/tmp/atkm_list.$$");
					}
					$new_url = "tests_results.pl?details=$results_dir".($recursive_dirs ? "&analysis=1" : "");
				}
			}
		}
		else {
			$error = "Cannot open file $save_profile_name for writing!<br />$!";
		}
	}
}

if (defined($error)) {
	escape($error);
	print "HTTP/1.0 302 Moved\n";
	print "Set-Cookie: last_error_text=$error; path=/;\n";
	print "Location: $conf_page\n\n";
}
else {
	print "HTTP/1.0 302 Moved\n";
	print "Location: $new_url\n\n";
}
